#ifndef COMIX_Run_Environment_H
#define COMIX_Run_Environment_H

#include "ATOOLS/Org/CXXFLAGS.H"

#include "MODEL/Main/Model_Base.H"
#include "BEAM/Main/Beam_Spectra_Handler.H"
#include "PDF/Main/ISR_Handler.H"
#include "COMIX/Run/ME_Handler.H"
#include "ATOOLS/Org/Data_Reader.H"
#include "ATOOLS/Org/Exception.H"

#include <map>
#include <string>
#include <iostream> 

namespace COMIX {

  struct isr {

    enum id {
      none            =  0,
      hard_process    =  1,
      hard_subprocess =  2,
      unknown         = 99
    };

  };// end of struct isr

  typedef std::map<isr::id,PDF::ISR_Handler*> ISR_Handler_Map;

  class Environment: public ATOOLS::Terminator_Object {
  private :

    std::string m_path, m_file, m_pdflib;
    std::string m_modeldat, m_beamdat, m_isrdat, m_medat;
    std::string m_processesdat, m_selectordat;

    double              m_bunch_splimits[2];
    ATOOLS::Flavour     m_bunch_particles[2];

    ATOOLS::Data_Reader *p_dataread;

    bool m_savestatus;

    MODEL::Model_Base          * p_model;
    BEAM::Beam_Spectra_Handler * p_beamspectra;
    ISR_Handler_Map              m_isrhandlers;
    MEHandlersMap                m_mehandlers;

    void ShowParameterSyntax();
    void LoadLibraries() const;

    bool InitializeTheModel();
    bool InitializeTheBeams();
    bool InitializeThePDFs();
    bool InitializeTheMatrixElements();

    bool ExtractValArg
    (std::vector<std::string> &args,
     std::vector<std::string>::iterator &it,
     const std::string &arg,const std::string &tag,
     const std::string &def=std::string("")) const;
    int  ExtractCommandLineParameters(int argc,char * argv[]);
    bool CheckBeamISRConsistency();
    void SetScaleFactors();
    void PrepareTerminate();
    void SetFileNames();

  public :

    Environment(int argc,char * argv[]);

    ~Environment();

    bool Initialize();

    bool CalculateTheHardProcesses();

    ME_Handler *const GetMatrixElementHandler(std::string="SignalMEs");

  };// end of class Environment

}// end of namespace COMIX

#endif
