#include "COMIX/Run/Environment.H"
#include "COMIX/Run/ME_Handler.H"
#include "ATOOLS/Org/Message.H"
#include "ATOOLS/Org/Exception.H"
#include "ATOOLS/Org/Library_Loader.H"
#include "ATOOLS/Org/CXXFLAGS.H"
#include "ATOOLS/Org/CXXFLAGS_PACKAGES.H"

using namespace COMIX;

#ifdef FC_DUMMY_MAIN
extern "C" int FC_DUMMY_MAIN() { return 1; }
#endif

int main(int argc,char* argv[])
{
  ATOOLS::exh->Init();
  ATOOLS::exh->SetProgramName("Comix");
  std::set_terminate(ATOOLS::Terminate);
  std::set_unexpected(ATOOLS::Terminate);
  signal(SIGSEGV,ATOOLS::SignalHandler);
  signal(SIGINT,ATOOLS::SignalHandler);
  signal(SIGPIPE,ATOOLS::SignalHandler);
  signal(SIGBUS,ATOOLS::SignalHandler);
  signal(SIGFPE,ATOOLS::SignalHandler);
  signal(SIGABRT,ATOOLS::SignalHandler);
  signal(SIGTERM,ATOOLS::SignalHandler);
  signal(SIGXCPU,ATOOLS::SignalHandler);
  try {
    // start main program
    ATOOLS::s_loader = new ATOOLS::Library_Loader();
    Environment env(argc, argv);
    if (env.Initialize()) {
      if (!env.GetMatrixElementHandler()->CalculateTotalXSecs())
	THROW(critical_error,"No process calculated");
    }
    delete ATOOLS::s_loader;
    return 0;
    // end main program
  }
  catch (ATOOLS::Exception exception) {
    exception.UpdateLogFile();
    msg_Error()<<exception<<std::endl;
    std::terminate();
  }
  catch (std::exception exception) {
    std::cout<<"Sherpa: throws std::exception "<<exception.what()<<" ..."<<std::endl;
    abort();
  }
}



